from typing import Tuple
from pydantic import BaseModel, validator, StrictStr


class AftereHookValidator(BaseModel):
    names: Tuple[StrictStr, ...]

    @validator('names')
    def validate_names(cls, vals):
        assert len(vals) > 0
        assert len(vals) == len(set(vals))
    


class AfterValidator:

    def __init__(__pydantic_self__, *names: Tuple[str, ...]) -> None:
        AftereHookValidator(
            names=names
        )


import uuid
from typing import List
from hedra.logging import HedraLogger
from hedra.reporting.events.types.base_event import BaseEvent


try:
    from hedra.reporting.types.statsd import StatsD
    from aio_statsd import TelegrafStatsdClient
    from .teleraf_statsd_config import TelegrafStatsDConfig
    has_connector = True

except Exception:
    from hedra.reporting.types.empty import Empty as StatsD
    TelegrafStatsDConfig=None
    TelegrafStatsdClient = None
    has_connector = False


class TelegrafStatsD(StatsD):

    def __init__(self, config: TelegrafStatsDConfig) -> None:
        super().__init__(config)
        self.connection = TelegrafStatsdClient(
            host=self.host,
            port=self.port
        )

        self.types_map = {
            'total': 'increment',
            'succeeded': 'increment',
            'failed': 'increment',
            'median': 'gauge',
            'mean': 'gauge',
            'variance': 'gauge',
            'stdev': 'gauge',
            'minimum': 'gauge',
            'maximum': 'gauge',
            'quantiles': 'gauge'
        }

        self._update_map = {
            'count': lambda: NotImplementedError('TelegrafStatsD does not support counts.'),
            'gauge': self.connection.gauge,
            'sets': lambda: NotImplementedError('TelegrafStatsD does not support sets.'),
            'increment': self.connection.increment,
            'histogram': self.connection.histogram,
            'distribution': self.connection.distribution,
            'timer': self.connection.timer
        }

        self.session_uuid = str(uuid.uuid4())
        self.metadata_string: str = None
        self.logger = HedraLogger()
        self.logger.initialize()

        self.statsd_type = 'TelegrafStatsD'

    async def submit_events(self, events: List[BaseEvent]):
        
        await self.logger.filesystem.aio['hedra.reporting'].info(f'{self.metadata_string} - Submitting Events to {self.statsd_type}')

        for event in events:
            time_update_function = self._update_map.get('gauge')
            time_update_function(f'{event.name}_time', event.time)
            
            if event.success:
                success_update_function = self._update_map.get('increment')
                success_update_function(f'{event.name}_success', 1)
            
            else:
                failed_update_function = self._update_map.get('increment')
                failed_update_function(f'{event.name}_failed', 1)

        await self.logger.filesystem.aio['hedra.reporting'].info(f'{self.metadata_string} - Submitted Events to {self.statsd_type}')



from typing import Type, Callable, Awaitable, Any, Dict, Optional
from hedra.core.graphs.hooks.hook_types.hook_type import HookType
from hedra.core.graphs.simple_context import SimpleContext
from .hook import Hook
from .hook_metadata import HookMetadata


class SetupHook(Hook):

    def __init__(
        self, 
        name: str, 
        shortname: str, 
        call: Callable[..., Awaitable[Any]], 
        key: Optional[str]=None,
        metadata: Dict[str, Any]={}
    ) -> None:
        super().__init__(
            name, 
            shortname, 
            call, 
            hook_type=HookType.SETUP
        )

        self.key = key
        self.metadata = HookMetadata(**metadata)

    async def call(self, context: SimpleContext):
        results = await self._call()

        if self.key:
            context[self.key] = results


        return context


import functools
from .hook_type import HookType
from hedra.core.graphs.hooks.registry.registrar import registrar


@registrar(HookType.TEARDOWN)
def teardown(key: str=None, metadata={}):

    def wrapper(func):

        @functools.wraps(func)
        async def decorator(*args, **kwargs):
            return await func(*args, **kwargs)

        return decorator
    return wrapper


from typing import Tuple, Optional
from pydantic import BaseModel, Field, validator, StrictStr


class ValidateHookValidator(BaseModel):
    stage: StrictStr=Field(..., min_length=1)
    names: Optional[Tuple[StrictStr, ...]]

    @validator('names')
    def validate_names(cls, vals):
        assert len(vals) == len(set(vals)), "Names must be unique for @validate() hook."


class ValidateValidator:

    def __init__(__pydantic_self__, stage: str, names: Optional[Tuple[str, ...]]=()) -> None:
        ValidateHookValidator(
            stage=stage,
            names=names
        )
    


from typing import Tuple
from pydantic import BaseModel, validator, StrictStr, StrictBool


class EventHookValidator(BaseModel):
    names: Tuple[StrictStr, ...]
    pre: StrictBool=False

    @validator('names')
    def validate_names(cls, vals):
        assert len(vals) > 0
        assert len(vals) == len(set(vals))


class EventValidator:

    def __init__(__pydantic_self__, *names: Tuple[str, ...], pre: bool=False) -> None:
        EventHookValidator(
            names=names,
            pre=pre
        )



from typing import Optional
from pydantic import BaseModel, Field, StrictStr


class ContextHookValidator(BaseModel):
    store_key: StrictStr=Field(..., min_length=1)
    load_key: Optional[StrictStr]=Field(None, min_length=1)


class ContextValidator:

    def __init__(self, store_key: str, load_key: Optional[str]=None) -> None:
        ContextHookValidator(
            store_key=store_key,
            load_key=load_key
        )



from pydantic import BaseModel, Field, StrictStr


class MetricHookValidator(BaseModel):
    group: StrictStr=Field(..., min_length=1)


class MetricValidator:

    def __init__(__pydantic_self__,  group: str) -> None:
        MetricHookValidator(
            group=group
        )



from pydantic import BaseModel, Field, StrictStr


class SaveHookValidator(BaseModel):
    key: StrictStr=Field(..., min_length=1)
    checkpoint_filepath: StrictStr=Field(..., min_length=1)


class SaveValidator:

    def __init__(__pydantic_self__, key: str, checkpoint_filepath: str) -> None:
        SaveHookValidator(
            key=key,
            checkpoint_filepath=checkpoint_filepath
        )